# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from collections import abc
from collections.abc import Mapping
from typing import Generic, TypeVar

from ikigai.utils.protocols import Named

VT = TypeVar("VT", bound=Named)


class NamedMapping(Generic[VT], Mapping[str, VT]):
    def __init__(self, mapping: Mapping[str, VT]) -> None:
        self._mapping = dict(mapping)

    def __getitem__(self, key: str) -> VT:
        matches = [item for item in self._mapping.values() if item.name == key]
        if not matches:
            raise KeyError(key)
        if len(matches) > 1:
            error_msg = (
                f'Multiple({len(matches)}) items with name: "{key}", '
                f'use get_id(id="...") to disambiguiate between {matches}'
            )
            raise KeyError(error_msg)
        return matches[0]

    def __iter__(self) -> abc.Iterator[str]:
        return iter({item.name for item in self._mapping.values()})

    def __len__(self) -> int:
        return len(self._mapping)

    def __repr__(self) -> str:
        return repr(self._mapping)

    def get_id(self, id: str) -> VT:
        return self._mapping[id]



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

import math
import random
from typing import Any, Callable

import pandas as pd
import pytest
from faker import Faker
from ikigai import Ikigai

ColumnNames = list[str]
ColumnGenerator = Callable[[], Any]
ColumnGenerators = list[ColumnGenerator]
TableGenerator = tuple[ColumnNames, ColumnGenerators]


def create_table_generator(fake: Faker, k: int) -> TableGenerator:
    """
    Return a table generator that generates a table with randomly selected columns
    where, k <= number of columns <= 2k
    """
    available_column_generators: list[list[tuple[str, ColumnGenerator]]] = [
        [("name", fake.name), ("age", lambda: random.randint(20, 90))],
        [("ssn", fake.ssn)],
        [
            ("lat", lambda: float(fake.latitude())),
            ("lon", lambda: float(fake.longitude())),
        ],
        [("date", lambda: str(fake.date_time()))],
        [("email", fake.free_email)],
        [("work_email", fake.company_email)],
        [("job", fake.job), ("salary", fake.pricetag)],
    ]

    column_generators_selection = random.choices(available_column_generators, k=k)

    table_column_names: ColumnNames = []
    table_column_generators: ColumnGenerators = []
    for idx, column_generators in enumerate(column_generators_selection, start=1):
        for column_name, column_generator in column_generators:
            table_column_names.append(f"{column_name}-{idx}")
            table_column_generators.append(column_generator)
    return table_column_names, table_column_generators


@pytest.fixture
def ikigai(cred: dict[str, Any]) -> Ikigai:
    return Ikigai(**cred)


@pytest.fixture
def app_name(random_name: str) -> str:
    return f"proj-{random_name}"


def generate_df(table_generator: TableGenerator, num_rows: int) -> pd.DataFrame:
    column_names, column_generators = table_generator
    rows = [[gen() for gen in column_generators] for _ in range(num_rows)]
    return pd.DataFrame(data=rows, columns=column_names)


@pytest.fixture
def df1(faker: Faker) -> pd.DataFrame:
    num_generator_selections = random.randint(1, 10)
    num_rows = math.ceil(random.triangular(1, 100))
    table_generator = create_table_generator(faker, num_generator_selections)
    return generate_df(table_generator=table_generator, num_rows=num_rows)


@pytest.fixture
def df2(faker: Faker) -> pd.DataFrame:
    num_generator_selections = random.randint(1, 10)
    num_rows = math.ceil(random.triangular(1, 100))
    table_generator = create_table_generator(faker, num_generator_selections)
    return generate_df(table_generator=table_generator, num_rows=num_rows)


@pytest.fixture
def dataset_name(random_name: str) -> str:
    return f"dats-{random_name}"



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from __future__ import annotations

import sys
from http import HTTPStatus
from typing import Any

import requests
from pydantic import ConfigDict
from pydantic.dataclasses import dataclass
from requests import Response

from ikigai.utils.compatibility import HTTPMethod


@dataclass
class Session:
    base_url: str
    session: requests.Session

    __pydantic_config__ = ConfigDict(arbitrary_types_allowed=True)

    def request(
        self,
        method: HTTPMethod,
        path: str,
        params: dict[str, str] | None = None,
        json: dict | None = None,
    ) -> Response:
        url = f"{self.base_url}{path}"
        resp = self.session.request(
            method=method,
            url=url,
            params=params,
            json=json,
        )
        if resp.status_code < HTTPStatus.BAD_REQUEST:
            return resp
        elif resp.status_code < HTTPStatus.INTERNAL_SERVER_ERROR:
            # A 4XX error happened
            print(
                f"""{method} {path}\n"""
                f"""{resp.request.body!r}\n\n"""
                f"""{resp.raw.headers}\n"""
                f"""{resp.text}""",
                file=sys.stderr,
            )
            todo = "TODO: Add error reporting"
            raise NotImplementedError(todo)
        else:
            # A 5XX error happened
            print(
                f"""{method} {path}\n"""
                f"""{resp.request.body!r}\n\n"""
                f"""{resp.raw.headers}\n"""
                f"""{resp.text}""",
                file=sys.stderr,
            )
            todo = "TODO: Add error reporting"
            raise NotImplementedError(todo)
        return resp

    def get(self, path: str, params: dict[str, Any] | None = None) -> Response:
        return self.request(method=HTTPMethod.GET, path=path, params=params)

    def post(self, path: str, json: dict[Any, Any] | None = None) -> Response:
        return self.request(method=HTTPMethod.POST, path=path, json=json)

    def __del__(self) -> None:
        self.session.close()



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from contextlib import ExitStack

import pytest
from ikigai import Ikigai


def test_app_creation(ikigai: Ikigai, app_name: str, cleanup: ExitStack) -> None:
    apps = ikigai.apps()
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)

    app_dict = app.to_dict()
    apps_after_creation = ikigai.apps()

    with pytest.raises(KeyError):
        apps.get_id(app.app_id)
    assert app_dict["name"] == app_name
    assert app_dict["description"] == "A test app"
    assert apps_after_creation.get_id(app.app_id) is not None


def test_app_editing(ikigai: Ikigai, app_name: str, cleanup: ExitStack) -> None:
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)

    app.rename(f"updated {app_name}")
    app.update_description("An updated test app")

    app_after_edit = ikigai.apps().get_id(id=app.app_id)

    assert app_after_edit.name == app.name
    assert app_after_edit.description == app.description
    assert app_after_edit.name == f"updated {app_name}"
    assert app_after_edit.description == "An updated test app"


def test_app_describe_1(ikigai: Ikigai, app_name: str, cleanup: ExitStack) -> None:
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)

    description = app.describe()
    assert description is not None
    assert "app" in description
    assert description["app"]["name"] == app_name
    assert "components" in description
    components = description["components"]
    assert "charts" in components and components["charts"] == []
    assert "connectors" in components and components["connectors"] == []
    assert "dashboards" in components and components["dashboards"] == []
    assert "datasets" in components and components["datasets"] == []
    assert "databases" in components and components["databases"] == []
    assert "pipelines" in components and components["pipelines"] == []
    assert "models" in components and components["models"] == []
    assert "external_resources" in components and components["external_resources"] == []



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

import random
import sys
from collections.abc import Generator
from contextlib import ExitStack
from pathlib import Path
from typing import Any

import pytest

# Multiple python version compatible import for reading toml
if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib


def read_credentials(env_file: Path) -> dict[str, Any]:
    with env_file.open("rb") as env:
        users: dict[str, dict[str, str]] = tomllib.load(env)["credentials"]["users"]
    return {"ids": users.keys(), "params": users.values()}


@pytest.fixture(**read_credentials(Path("./test-env.toml")))
def cred(request) -> dict[str, str]:
    return request.param


ALPHABET = "abcdefghijklmnopqrstuvwxyz0123456789-"


@pytest.fixture
def random_name() -> str:
    name_length = int(random.triangular(low=5, high=20, mode=20))
    return "".join(random.choices(ALPHABET, k=name_length))


@pytest.fixture
def cleanup() -> Generator[ExitStack, None, None]:
    exit_stack = ExitStack()
    try:
        yield exit_stack
    finally:
        exit_stack.close()



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from contextlib import ExitStack

import pandas as pd
import pytest
from ikigai.ikigai import Ikigai


def test_dataset_creation(
    ikigai: Ikigai,
    app_name: str,
    dataset_name: str,
    df1: pd.DataFrame,
    cleanup: ExitStack,
) -> None:
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)

    datasets = app.datasets()
    assert len(datasets) == 0

    dataset = app.dataset.new(name=dataset_name).df(df1).build()
    cleanup.callback(dataset.delete)

    with pytest.raises(KeyError):
        datasets.get_id(dataset.dataset_id)

    datasets_after_creation = app.datasets()
    assert len(datasets_after_creation) == 1

    dataset_dict = dataset.to_dict()
    assert dataset_dict["name"] == dataset_name


def test_dataset_editing(
    ikigai: Ikigai,
    app_name: str,
    dataset_name: str,
    df1: pd.DataFrame,
    df2: pd.DataFrame,
    cleanup: ExitStack,
) -> None:
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)
    dataset = app.dataset.new(name=dataset_name).df(df1).build()
    cleanup.callback(dataset.delete)

    dataset.rename(f"updated {dataset_name}")
    dataset.edit_data(df2)

    dataset_after_edit = app.datasets().get_id(dataset.dataset_id)
    round_trip_df2 = dataset_after_edit.df()

    assert dataset_after_edit.name == dataset.name
    assert dataset_after_edit.name == f"updated {dataset_name}"
    assert df2.columns.equals(round_trip_df2.columns)
    pd.testing.assert_frame_equal(
        df2, round_trip_df2, check_dtype=False, check_exact=False
    )


def test_dataset_download(
    ikigai: Ikigai,
    app_name: str,
    dataset_name: str,
    df1: pd.DataFrame,
    cleanup: ExitStack,
) -> None:
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)
    dataset = app.dataset.new(name=dataset_name).df(df1).build()
    cleanup.callback(dataset.delete)

    round_trip_df1 = dataset.df()
    assert len(df1) == len(round_trip_df1)
    assert df1.columns.equals(round_trip_df1.columns)

    # v. helpful debug message when the test fails
    print(
        df1.dtypes, df1.head(), round_trip_df1.dtypes, round_trip_df1.head(), sep="\n"
    )

    pd.testing.assert_frame_equal(
        df1, round_trip_df1, check_dtype=False, check_exact=False
    )


def test_dataset_describe(
    ikigai: Ikigai,
    app_name: str,
    dataset_name: str,
    df1: pd.DataFrame,
    cleanup: ExitStack,
) -> None:
    app = ikigai.app.new(name=app_name).description("A test app").build()
    cleanup.callback(app.delete)
    dataset = app.dataset.new(name=dataset_name).df(df1).build()
    cleanup.callback(dataset.delete)

    description = dataset.describe()
    assert description is not None
    assert "dataset" in description
    assert description["dataset"]["name"] == dataset_name
    assert description["dataset"]["project_id"] == app.app_id
    assert description["dataset"]["directory"] is not None
    assert description["dataset"]["directory"]["type"] == "DATASET"



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from __future__ import annotations

import sys

# Multiple python version compatible import for Self
if sys.version_info >= (3, 11):
    from typing import Self  # noqa: F401
else:
    from typing_extensions import Self  # noqa: F401

# Multiple python version compatible import for HTTPMethod
if sys.version_info >= (3, 11):
    from http import HTTPMethod
else:
    from types import SimpleNamespace

    HTTPMethod = SimpleNamespace(
        CONNECT="CONNECT",
        DELETE="DELETE",
        GET="GET",
        HEAD="HEAD",
        OPTIONS="OPTIONS",
        PATCH="PATCH",
        POST="POST",
        PUT="PUT",
        TRACE="TRACE",
    )



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from ikigai.components.app import App, AppBuilder
from ikigai.components.dataset import Dataset, DatasetBuilder

__all__ = ["App", "AppBuilder", "Dataset", "DatasetBuilder"]



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import BaseModel, EmailStr, Field

from ikigai import components
from ikigai.client.session import Session
from ikigai.utils.compatibility import Self
from ikigai.utils.named_mapping import NamedMapping
from ikigai.utils.protocols import Directory


class AppBuilder:
    _name: str
    _description: str
    _directory: dict[str, str]
    _icon: str
    _images: list[str]
    __session: Session

    def __init__(self, session: Session) -> None:
        self.__session = session
        self._name = ""
        self._description = ""
        self._directory = {}
        self._icon = ""
        self._images = []

    def new(self, name: str) -> Self:
        self._name = name
        return self

    def description(self, description: str) -> Self:
        self._description = description
        return self

    def directory(self, directory: Directory) -> Self:
        self._directory = {
            "directory_id": directory.directory_id,
            "type": directory.type,
        }
        return self

    def build(self) -> App:
        resp = self.__session.post(
            path="/component/create-project",
            json={
                "project": {
                    "name": self._name,
                    "description": self._description,
                    "directory": self._directory,
                },
            },
        ).json()
        app_id = resp["project_id"]
        resp = self.__session.get(
            path="/component/get-project", params={"project_id": app_id}
        ).json()
        app = App.from_dict(data=resp["project"], session=self.__session)
        return app


class App(BaseModel):
    app_id: str = Field(validation_alias="project_id")
    name: str
    owner: EmailStr
    description: str
    created_at: datetime
    modified_at: datetime
    last_used_at: datetime
    __session: Session

    @classmethod
    def from_dict(cls, data: dict, session: Session) -> Self:
        self = cls.model_validate(data)
        self.__session = session
        return self

    """
    Operations on App
    """

    def to_dict(self) -> dict:
        return {
            "app_id": self.app_id,
            "name": self.name,
            "owner": self.owner,
            "description": self.description,
            "created_at": self.created_at,
            "modified_at": self.modified_at,
            "last_used_at": self.last_used_at,
        }

    def delete(self) -> None:
        self.__session.post(
            path="/component/delete-project",
            json={"project": {"project_id": self.app_id}},
        )
        return None

    def rename(self, name: str) -> Self:
        _ = self.__session.post(
            path="/component/edit-project",
            json={"project": {"project_id": self.app_id, "name": name}},
        )
        # TODO: handle error case, currently it is a raise NotImplemented from Session
        self.name = name
        return self

    def update_description(self, description: str) -> Self:
        _ = self.__session.post(
            path="/component/edit-project",
            json={"project": {"project_id": self.app_id, "description": description}},
        ).json()
        # TODO: handle error case, currently it is a raise NotImplemented from Session
        self.description = description
        return self

    def describe(self) -> dict:
        response: dict[str, Any] = self.__session.get(
            path="/component/get-components-for-project",
            params={"project_id": self.app_id},
        ).json()

        # Combine components information with app info
        return_value = {
            "app": self.to_dict(),
            "components": response["project_components"][self.app_id],
        }

        return return_value

    """
    Access Components in the App
    """

    def datasets(self) -> NamedMapping[components.Dataset]:
        resp = self.__session.get(
            path="/component/get-datasets-for-project",
            params={"project_id": self.app_id},
        ).json()
        datasets = {
            dataset.dataset_id: dataset
            for dataset in (
                components.Dataset.from_dict(data=dataset_dict, session=self.__session)
                for dataset_dict in resp["datasets"]
            )
        }

        return NamedMapping(datasets)

    @property
    def dataset(self) -> components.DatasetBuilder:
        return components.DatasetBuilder(session=self.__session, app_id=self.app_id)



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from __future__ import annotations

import io
import math
import time
from datetime import datetime
from enum import Enum
from http import HTTPStatus
from pathlib import Path
from typing import Any

import pandas as pd
import requests
from pydantic import BaseModel, Field

from ikigai.client.session import Session
from ikigai.utils.compatibility import Self
from ikigai.utils.protocols import Directory

CHUNK_SIZE = int(50e6)  # 50 MB


def __upload_data(
    session: Session,
    app_id: str,
    dataset_id: str,
    data: bytes,
    filename: str,
    chunk_size: int,
) -> None:
    num_parts = math.ceil(len(data) / chunk_size)

    resp = session.get(
        path="/component/get-dataset-multipart-upload-urls",
        params={
            "dataset_id": dataset_id,
            "project_id": app_id,
            "filename": filename,
            "number_of_parts": num_parts,
        },
    ).json()

    content_type = resp["content_type"]
    upload_urls: dict[int, str] = {
        int(chunk_idx): upload_url for chunk_idx, upload_url in resp["urls"].items()
    }
    upload_id = resp["upload_id"]
    etags: dict[int, str] = {}

    try:
        with requests.session() as request:
            request.headers.update(
                {"Content-Type": content_type, "Cache-Control": "no-cache"}
            )
            for chunk_idx, upload_url in upload_urls.items():
                chunk_start, chunk_end = (
                    (chunk_idx - 1) * chunk_size,
                    chunk_idx * chunk_size,
                )
                chunk = data[chunk_start:chunk_end]
                resp = request.put(url=upload_url, data=chunk)
                assert resp.status_code == HTTPStatus.OK

                # Get etags from response header
                etags[chunk_idx] = resp.headers.get("ETag")

    except Exception:
        session.post(
            path="/component/complete-dataset-multipart-upload",
            json={
                "dataset": {
                    "dataset_id": dataset_id,
                    "project_id": app_id,
                    "filename": filename,
                },
                "abort": True,
                "upload_id": upload_id,
            },
        )
        raise

    # Complete Dataset upload
    session.post(
        path="/component/complete-dataset-multipart-upload",
        json={
            "dataset": {
                "dataset_id": dataset_id,
                "project_id": app_id,
                "filename": filename,
            },
            "abort": False,
            "upload_id": upload_id,
            "etags": etags,
        },
    )


def _upload_data(
    session: Session, app_id: str, dataset_id: str, name: str, data: bytes
) -> None:
    assert data is not None
    filename = f"{name}.csv"

    __upload_data(
        session=session,
        app_id=app_id,
        dataset_id=dataset_id,
        data=data,
        filename=filename,
        chunk_size=CHUNK_SIZE,
    )

    upload_completion_time = time.time()
    session.get(
        path="/component/verify-dataset-upload",
        params={"dataset_id": dataset_id, "filename": filename},
    )

    dataset_status: str = "RUNNING"
    while dataset_status == "RUNNING":
        # Block thread while dataset is still being processed
        time.sleep(0.25)
        dataset_status = session.get(
            path="/component/confirm-dataset-upload",
            params={"dataset_id": dataset_id, "project_id": app_id},
        ).json()["status"]

    if dataset_status != "SUCCESS":
        error_msg = f"Dataset upload failed, upload ended with status {dataset_status}"
        raise RuntimeError(error_msg)

    dataset_logs = session.get(
        path="/component/get-dataset-log",
        params={"dataset_id": dataset_id, "project_id": app_id, "limit": 5},
    ).json()["dataset_log"]
    dataset_verified = [
        log
        for log in dataset_logs
        if log["status"] == "SUCCESS"
        and int(log["timestamp"]) > upload_completion_time
        and log["job_type"] == "UPLOAD_DATASET"
    ]
    if not dataset_verified:
        # TODO: Improve Error
        error_msg = "Dataset failed to verify"
        raise RuntimeError(error_msg, dataset_logs)


class DatasetBuilder:
    _app_id: str
    _name: str
    _data: bytes | None
    _directory: dict[str, str]
    __session: Session

    def __init__(self, session: Session, app_id: str) -> None:
        self.__session = session
        self._app_id = app_id
        self._name = ""
        self._data = None
        self._directory = {}

    def new(self, name: str) -> Self:
        self._name = name
        return self

    def df(self, data: pd.DataFrame) -> Self:
        buffer = io.BytesIO()
        data.to_csv(buffer, index_label=False, index=False)
        self._data = bytearray(buffer.getvalue())
        return self

    def csv(self, file: Path | str) -> Self:
        if isinstance(file, str):
            file = Path(file)
        with file.open("rb") as fp:
            self._data = bytearray(fp.read())
        return self

    def directory(self, directory: Directory) -> Self:
        self._directory = {
            "directory_id": directory.directory_id,
            "type": directory.type,
        }
        return self

    def build(self) -> Dataset:
        if self._data is None:
            error_msg = "Dataset is empty"
            raise ValueError(error_msg)

        resp = self.__session.post(
            path="/component/create-dataset",
            json={
                "dataset": {
                    "project_id": self._app_id,
                    "name": self._name,
                    "directory": self._directory,
                },
            },
        ).json()
        dataset_id = resp["dataset_id"]

        try:
            _upload_data(
                session=self.__session,
                app_id=self._app_id,
                dataset_id=dataset_id,
                name=self._name,
                data=self._data,
            )
        except Exception:
            # Delete created record and re-raise
            self.__session.post(
                path="/component/delete-dataset",
                json={
                    "dataset": {"project_id": self._app_id, "dataset_id": dataset_id}
                },
            )
            raise

        # Populate Dataset object
        resp = self.__session.get(
            path="/component/get-dataset", params={"dataset_id": dataset_id}
        ).json()
        dataset = Dataset.from_dict(data=resp["dataset"], session=self.__session)
        return dataset


class DataType(str, Enum):
    NUMERIC = "NUMERIC"
    TEXT = "TEXT"
    CATEGORICAL = "CATEGORICAL"
    TIME = "TIME"


class ColumnDataType(BaseModel):
    data_type: DataType
    data_formats: dict[str, str]


class Dataset(BaseModel):
    app_id: str = Field(validation_alias="project_id")
    dataset_id: str
    name: str
    filename: str
    file_extension: str
    data_types: dict[str, ColumnDataType]
    size: int
    created_at: datetime
    modified_at: datetime
    __session: Session

    @classmethod
    def from_dict(cls, data: dict, session: Session) -> Self:
        self = cls.model_validate(data)
        self.__session = session
        return self

    def to_dict(self) -> dict:
        return {
            "dataset_id": self.dataset_id,
            "name": self.name,
            "filename": self.filename,
            "file_extension": self.file_extension,
            "data_types": self.data_types,
            "size": self.size,
            "created_at": self.created_at,
            "modified_at": self.modified_at,
        }

    def delete(self) -> None:
        self.__session.post(
            path="/component/delete-dataset",
            json={"dataset": {"dataset_id": self.dataset_id}},
        )
        return None

    def rename(self, name: str) -> Self:
        _ = self.__session.post(
            path="/component/edit-dataset",
            json={
                "dataset": {
                    "project_id": self.app_id,
                    "dataset_id": self.dataset_id,
                    "name": name,
                }
            },
        )
        # TODO: handle error case, currently it is a raise NotImplemented from Session
        self.name = name
        return self

    def df(self, **parser_options) -> pd.DataFrame:
        resp = self.__session.get(
            path="/component/get-dataset-download-url",
            params={
                "dataset_id": self.dataset_id,
                "project_id": self.app_id,
            },
        ).json()
        dataset_url = resp["url"]
        return pd.read_csv(dataset_url, **parser_options)

    def edit_data(self, data: pd.DataFrame) -> None:
        buffer = io.BytesIO()
        data.to_csv(buffer, index_label=False, index=False)

        _upload_data(
            session=self.__session,
            app_id=self.app_id,
            dataset_id=self.dataset_id,
            name=self.name,
            data=bytearray(buffer.getvalue()),
        )

    def describe(self) -> dict:
        response: dict[str, Any] = self.__session.get(
            path="/component/get-dataset",
            params={"dataset_id": self.dataset_id},
        ).json()

        return response



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

import requests
from pydantic import AnyUrl, EmailStr, Field
from pydantic.dataclasses import dataclass

from ikigai import components
from ikigai.client.session import Session
from ikigai.utils.named_mapping import NamedMapping


@dataclass
class Ikigai:
    user_email: EmailStr
    api_key: str = Field(repr=False)
    base_url: AnyUrl = Field(default=AnyUrl("https://api.ikigailabs.io"))
    __session: Session = Field(init=False)

    def __post_init__(self) -> None:
        session = requests.Session()
        session.headers.update({"user": self.user_email, "api-key": self.api_key})
        self.__session = Session(base_url=str(self.base_url), session=session)

    def apps(self) -> NamedMapping[components.App]:
        resp = self.__session.get("/component/get-projects-for-user").json()
        apps = {
            app.app_id: app
            for app in (
                components.App.from_dict(data=app_dict, session=self.__session)
                for app_dict in resp["projects"]
            )
        }

        return NamedMapping(apps)

    @property
    def app(self) -> components.AppBuilder:
        return components.AppBuilder(session=self.__session)



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from typing import Protocol


class Named(Protocol):
    name: str


class Directory(Protocol):
    directory_id: str
    type: str



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from typing import Any

import pytest
from ikigai import Ikigai


def test_client_init(cred: dict[str, Any]) -> None:
    ikigai = Ikigai(**cred)
    assert ikigai


def test_client_apps(cred: dict[str, Any]) -> None:
    ikigai = Ikigai(**cred)
    apps = ikigai.apps()
    assert len(apps) > 0


def test_client_app_get_item(cred: dict[str, Any]) -> None:
    ikigai = Ikigai(**cred)
    apps = ikigai.apps()
    with pytest.raises(KeyError):
        apps["Testing"]
